*===================================================================;
*Dec 31, 2011: (1) streamline data_prep_by_product_update4.sas code ;
*                  located at F:\china project\gravity_model.sas and;
*                  reg_data_updated.sas code located at             ;
*                  F:\stata\gravity\product_regression_updated2     ;
*              (2) pool all accessory codes to global_macros_prep   ;
*              (3) do only for export                               ;
*March 22, 2012: (1), add OLS measures of productitity and lags     ;
*===================================================================;
%let dir_source=F:\china project;
libname out "F:\stata\gravity\product_regression_updated4\sas_code\data";
libname output "&dir_source\sas_output_data";
libname link   "&dir_source\export_import_decision_and_performance\transition";  

*==============================================================================;
*Merge export_by_product data with crosswalk and then with firm characteristics;
*==============================================================================;
proc sql;
 create table export_by_product as
 select a.party_id,
        a.year,
        a.quantity,
		a.value,
        a.valperunit_mean,
        a.hs8,
		a.hs96_01,
		a.transportation_mode,
        a.type,
        a.unit_count,
        a.isocode, 
		a.count_hs,
		a.count_isocode,
		a.core_product,
        a.count_hs_import,
		a.count_isocode_import,  
        a.wgt_by_isocode_hs,
		a.wgt_by_hs,
        c.frdm_panel_new
 from output.export_by_product as a, link.trade_id_panel_newid as c
 where a.party_id=c.party_id;
quit; 

%let firm_characteristics=l_labor l_age l_wage cic_adj ownership l_sale l_tvc l_total_asset new_product_pct;
data firm_characteristics;
    set output.firm_characteristics;
	l_k_over_labor=l_k - l_labor;
	keep frdm_panel_new year l_k_over_labor &firm_characteristics;
run; 

data productivity_measures (rename=(tfp_ols_value_added=tfp_ols_vad));
    set  output.productivity_measures;
    keep frdm_panel_new year tfp_lp_value_added l_vad tfp_ols_value_added tfp_ols_output;
run;

proc sort data=export_by_product   tagsort; by frdm_panel_new year; run;
proc sort data=firm_characteristics tagsort; by frdm_panel_new year; run;
proc sort data=productivity_measures tagsort; by frdm_panel_new year; run; 

*=======================================================================================;
*March 22, 2012: the lagged versions of tfp_ols_vad and tfp_ols_output were not included;
*                in output.lag_productivity_measures, here I need to add them back      ;
*=======================================================================================;
data lag_productivity_measures1;
   set productivity_measures;
     lag_frdm_panel_new     =lag1(frdm_panel_new);
	 lag_year               =lag1(year);
     lag_tfp_ols_vad_tmp    =lag1(tfp_ols_vad);
	 lag_tfp_ols_output_tmp =lag1(tfp_ols_output);
	 if year=lag_year+1 and lag_frdm_panel_new=frdm_panel_new then do;
       lag_tfp_ols_vad     =lag_tfp_ols_vad_tmp;  
       lag_tfp_ols_output  =lag_tfp_ols_output_tmp;
     end;
	 keep year frdm_panel_new  lag_tfp_ols_vad lag_tfp_ols_output;
run;

data lag_productivity_measures (rename=(frdm_panel_new1=frdm_panel_new));
    set output.lag_productivity_measures;
	length frdm_panel_new1 $20.;
    frdm_panel_new1=frdm_panel_new;
	keep frdm_panel_new1 year lag_tfp_lp_value_added lag_l_vad;
run;

proc sort data=lag_productivity_measures  tagsort; by frdm_panel_new year; run; 
proc sort data=lag_productivity_measures1 tagsort; by frdm_panel_new year; run;

data lag_productivity_measures;
   merge lag_productivity_measures (in=a)
         lag_productivity_measures1;
   by frdm_panel_new year;
   if a then output;
run;

*================================================================================================;
*Notice: in F:\stata\gravity\product_regression_updated3\firm_characteristics.sas I newly created;
*        lag_firm_characteristics.sas7bdat in F:\stata\gravity\product_regression_updated3\data  ;
*        with a corrected version of l_wage (different from l_tvc now)- Dec 31, 2011!!! Huiwen   ;
*================================================================================================;
proc sort data=out.lag_firm_characteristics  out=lag_firm_characteristics  tagsort; by frdm_panel_new year; run;   


*=================================================================================;
*(1), Add 2005 and 2006 R&D and 2004-2006 AD variables, defines ad_intensity (for ;
*     04-06 only) and rd_intensity (for 05 and 06 only)                           ;                                                             ; 
*(2), add two ownership variables (1), to separate FIE, (2), to separate DOM.     ;
*     Ge Ying: Among FIEs, 330 is WFOEs, others are joint ventures.               ; 
*=================================================================================;
%include "&dir_source\Panel9807SAS\firm1999_update.sas"/source2;
data rd_ad_ownership (rename=(ownership_zhuchun=ownership1));
  length ownership_zhuchun ownership_geying $4.;
   set output.panel_pool_98_07_updated;
   if registration_code in (310,320,340) then ownership_geying='FIEJ';
   if registration_code=330               then ownership_geying='FIEW';
   if registration_code in (210,220,230,240) then ownership_geying='HMT';
   if registration_code in (110,151,141) then ownership_geying='SOE';
   if registration_code in (120,142) then ownership_geying='COE';
   if registration_code in (171,172,173,174) then ownership_geying='PRI';
   if registration_code in (130,143,149,159,160,190) then ownership_geying='DOM';
   *Zhu Chun: Use an aggregate classification for domestic firms because they have similar trade patterns;
   if ownership_geying in ('FIEJ','FIEW','HMT') then ownership_zhuchun=ownership_geying;
      else ownership_zhuchun='DOM';
    keep frdm_panel_new year  sale research advertisement ownership_zhuchun ;
run;

data rd_ad_ownership;
    set rd_ad_ownership;
	rd_dummy=(research>0);
    ad_dummy=(advertisement>0);
    if sale>0 then do;
       rd_intensity=research/sale; 
       ad_intensity=advertisement/sale; 
    end;
    keep frdm_panel_new year research advertisement rd_intensity ad_intensity ownership1;
run;

proc sort data=rd_ad_ownership nodupkey; by frdm_panel_new year; run; 

data export_by_product;
    merge export_by_product
          firm_characteristics (in=a)
          productivity_measures
          lag_firm_characteristics 
          lag_productivity_measures
		  rd_ad_ownership;
    by frdm_panel_new year;
	if a then output;
run;

*===============================================================;
*link &trade._by_product with exchange rate and gravity datasets;
*===============================================================;
proc sort data=output.realexchangerates out=realexchangerates; by isocode year; run;
proc sort data=output.gravity           out=gravity          ; by isocode year; run;

data exchangerate_gravity;
    merge realexchangerates
	      gravity;
    by isocode year;
run;

%let drop_lst= count_hs count_isocode wgt_by_isocode_hs exchange_real exchange_real_l1 exchange_nominal exchange_nominal_l1 
               neighbour common_language gdpcap_china gdp_china l_tvc transportation_mode core_product;

proc sort data=export_by_product tagsort; by isocode year; run;

data reg_data_export_by_product;
   merge export_by_product (in=a)
         exchangerate_gravity;
   by isocode year;
   if a then output;
   drop &drop_lst;
run;

%include "F:\stata\gravity\product_regression_updated3\sas_code\global_macros_prep.sas" /source2;

*==============================================================;
*add Median of R&D/sales or advertisement/sales by 4-digit CICs;
*==============================================================;
%rd_intensity_by_cic_adj;

proc sort data=reg_data_export_by_product out=reg_data_export_by_product tagsort; by frdm_panel_new year; run;
proc sort data=rd_intensity_by_cic_adj                                          ; by frdm_panel_new year; run;

data reg_data_export_by_product;
    merge reg_data_export_by_product (in=a)
          rd_intensity_by_cic_adj;
    by frdm_panel_new year; 
    if a then output;
run;

*=====================================;
*add sigma: elasticity of substitution;
*=====================================;
%sigma;

proc sort data=reg_data_export_by_product tagsort; by hs96_01; run;
proc sort data=sigma                             ; by hs96_01; run;

data reg_data_export_by_product;
    merge reg_data_export_by_product (in=a)
	      sigma;
    by hs96_01;
    if a then output;
run;

*===============================================================================================;
*Merge import_price_indexes_ind with reg_data_export_by_product. For capital imports,set related;
*dummy and price to previous year's results as long as any previous import none zero            ;                                        ;
*===============================================================================================;
%import_price_indexes_ind;

proc sort data=reg_data_export_by_product out=reg_data_export_by_product tagsort; by party_id year; run;
proc sort data=import_price_indexes_ind          ; by party_id year; run;

data reg_data_export_by_product;
     merge reg_data_export_by_product (in=a)
           import_price_indexes_ind;
     by party_id year;
     if a & year>=2000 & year<=2006 then output;
run;

data reg_data_export_by_product;
   set reg_data_export_by_product;
   by party_id;
   retain import_ind_capital_tmp import_price_capital_tmp;
   if first.party_id then do;  
      import_ind_capital_tmp  =.; 
      import_price_capital_tmp=.;
   end;
   if import_ind_capital  ^=. then import_ind_capital_tmp  =import_ind_capital;
   if import_ind_capital   =. then import_ind_capital      =import_ind_capital_tmp;
   if import_price_capital^=. then import_price_capital_tmp=import_price_capital;
   if import_price_capital =. then import_price_capital    =import_price_capital_tmp;
   drop import_ind_capital_tmp import_price_capital_tmp;
run;

%saveOne(data=reg_data_export_by_product);

*================================================================;
*add a balanced indicator: balanced_export, export>0 for all year;
*================================================================;
%add_balanced_ind;
data out.balanced_ind;
   set balanced_ind;
run;

proc sort data=balanced_ind                      ; by frdm_panel_new; run;
proc sort data=reg_data_export_by_product tagsort; by frdm_panel_new; run;

data reg_data_export_by_product;
     merge reg_data_export_by_product
	       balanced_ind;
     by frdm_panel_new;
run;

%saveOne(data=reg_data_export_by_product);

*===================================================================================;
*Add a corrected BEC code, Here I add it back by using following three links:       ;
*(1) BEC and HS6 for 1996 (2) BEC and HS6 for 2002 (3) HS6 for 1996 and HS6 for 2006;
*===================================================================================;
%add_corrected_bec;

data out.hs_bec_converter;
   set hs_bec_converter;
run;

proc sort data=hs_bec_converter nodupkey         ; by year hs96_01; run;
proc sort data=reg_data_export_by_product tagsort; by year hs96_01; run;

data  reg_data_export_by_product;
   merge reg_data_export_by_product (in=a)
         hs_bec_converter;
   by year hs96_01;
   if a then output;
run;

%saveOne(data=reg_data_export_by_product);

data out.reg_data_export_by_product;
    set reg_data_export_by_product;
run;

*=====================;
*add herfindal indexes;
*=====================;
%add_herfind_index;
 
data out.herfind_hs96_01_isocode;
     set herfind_hs96_01_isocode;
run;


 proc sort data=reg_data_export_by_product tagsort; by year hs96_01 isocode; run;
 proc sort data=herfind_hs96_01_isocode           ; by year hs96_01 isocode; run;

 data reg_data_export_by_product;
     merge reg_data_export_by_product (in=a)
	       herfind_hs96_01_isocode;
	by year hs96_01 isocode;
	if a then output;
run;;

*====================================================================;
*Add Rauch (1999) measure of product differentiation into the dataset;
*====================================================================; 
%hs96_01_to_rauch_classification;

data out.hs96_01_to_rauch_classification;
   set hs96_01_to_rauch_classification;
run;

proc sort data=hs96_01_to_rauch_classification nodupkey; by hs96_01; run;  /* 21 duplicated hs96_01 are repeated values, ok to deleted */
proc sort data=reg_data_export_by_product       tagsort; by hs96_01; run;

data reg_data_export_by_product;
    merge reg_data_export_by_product (in=a)
	      hs96_01_to_rauch_classification;
	by hs96_01;
	if a then output;
run;



*===================================================================;
*Create a variable two variables: (a) share_firm_hs96_01_isocode and;
*(b)lag_share_firm_hs96_01_isocode later to be added                ;
*===================================================================;
%lag_share_firm_hs96_01_isocode;
data out.lag_share_firm_hs96_01_isocode;
   set lag_share_firm_hs96_01_isocode;
run;

proc sort data=reg_data_export_by_product tagsort; by year frdm_panel_new hs96_01 isocode; run;
proc sort data=lag_share_firm_hs96_01_isocode    ; by year frdm_panel_new hs96_01 isocode; run;

data reg_data_export_by_product;
   merge reg_data_export_by_product (in=a)
         lag_share_firm_hs96_01_isocode; 
   by year frdm_panel_new hs96_01 isocode; 
   if a then output;
run;

*=====================================================================;
*add two variables generated in trade_regime/firm_attributes.sas      ;
*share_import_intermediate and share_import_capital. I copied it from ;
*"&dir_source\trade_regime\data\share_intermediate_capital.sas7bdat to;
*out.share_intermediate_capital                                       ;
*=====================================================================;
data share_intermediate_capital;
   set out.share_intermediate_capital;
run;

proc sort data=share_intermediate_capital        ; by frdm_panel_new year; run;
proc sort data=reg_data_export_by_product tagsort; by frdm_panel_new year; run;


data reg_data_export_by_product;
   merge reg_data_export_by_product (in=a)
         share_intermediate_capital;
	by frdm_panel_new year;
	if a then output;
run;

*=====================================================;
*Add worker skill variables to reg_data_export_product;
*=====================================================;
%skill_above_college;

proc sort data=panel_04 nodupkey            ; by year frdm_panel_new; run;  *only 7 dupkeys, ok to do nodupkey;
proc sort data=reg_data_export_by_product; by year frdm_panel_new; run;

%let drop_lst1=party_id count_hs_import count_isocode_import  advertisement research rauch_liberal;

data out.reg_data_export_by_product;
   merge reg_data_export_by_product (in=a)
         panel_04;
   by year frdm_panel_new;
   drop &drop_lst1;
   if a  & valperunit_mean^=. then output;
run;












