libname foreign "F:\china project\foreign_firm";
libname output  "F:\china project\sas_output_data";
%include "F:\china project\Panel9807SAS\firm1999_update.sas"/source2;
libname link   "F:\china project\export_import_decision_and_performance\transition";  
libname export "F:\stata\gravity\product_regression_updated2\data";

*================;
*Read retail data;
*================;
data foreign;
   set foreign.foreign;
*   keep name postcode phone area_code chairman ceo;
run;

proc sort data=foreign  nodupkey; by name; run;   
*================================================================================;
*(1): no duplicated name, but many empty legal_code so will reassign a foreign_id;
*(2): there are 1239 pairs of duplicated legal_code, visually checked, confirm   ;
*     that each represents different company, so reassign a foreign_id ok        ;
*================================================================================;
data foreign;
   set foreign;
   length foreign_id $10.;
   foreign_id=compress(put(_n_,8.));
run;

data foreign_tmp1;
   set foreign;
   length zip_pperson_ $30.
          company     $62.;
   company   =compress(name);
   if compress(chairman)^="" then pperson =compress(chairman);  
      else pperson=compress(ceo); 
      *============================================================================;
	  *assume chairman is the legal_name, or if no chairman then ceo the legal_name;
	  *============================================================================;
   zip_pperson_ =compress(postcode||pperson);

   tel1_tmp1     =0;
   tel1_tmp2     =compress(phone);
   if substr(tel1_tmp2,1,1)='0'   then tel1_tmp1=1;   
   if substr(tel1_tmp2,1,2)='00'  then tel1_tmp1=2;
   if substr(tel1_tmp2,1,3)='000' then tel1_tmp1=3;
   if tel1_tmp1=0 then tel1_=tel1_tmp2;
   if tel1_tmp1=1 then tel1_=substr(tel1_tmp2,2);
   if tel1_tmp1=2 then tel1_=substr(tel1_tmp2,3);
   if tel1_tmp1=3 then tel1_=substr(tel1_tmp2,4);

   if (company='' and pperson='' and tel1_='' and zip_pperson_='' ) then delete;
   keep foreign_id company tel1_ zip_pperson_;
run;


*================================================;
*Treat varialbes with irregular values as missing;
*================================================;
data foreign_tmp2;
   set foreign_tmp1;
   if substr(tel1_,1,1) in ('*','-','.') then tel1_=substr(tel1_,2); 
   if substr(tel1_,1,1) ='0'   or
      length(tel1_)      <5    or
      find(tel1_, 'E')    or
   substr(upcase(tel1_),1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345','XXXXX','*****') or
   substr(tel1_,1,1) ^in ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9')  then tel1_='';

   if substr(zip_pperson_,1,1)='0' then zip_pperson_=''; 
   if substr(zip_pperson_,1,1) ^in ('1','2','3','4','5','6','7','8','9') then zip_pperson_='';
   if length(zip_pperson_)<=6 then zip_pperson_='';
   if substr(company,1,1) in ('','"','-','.','0','[',']','`','h','i','I','n','r','s','t','y','z') then company=substr(company,2);
   if substr(company,1,1) in ('1','2','3','4','5','6','7','8','9') then company='';
   if substr(company,1,2) in ('??','?e','??','??,'??','??) then company=substr(company,3);
run;


data foreign_id      (rename=(company         =company_f
                        zip_pperson_   =zip_pperson_f
                        tel1_          =tel1_f
                        area_code_tel1_=area_code_tel1_f
                        tel2_          =tel2_f
                        area_code_tel2_=area_code_tel2_f));
   set foreign_tmp2;
   area_code_tel1_=tel1_;  /* define this variable indentical to tel1_ for opperational convenience when merged with trade data */ 
   tel2_          =tel1_;  /* define this variable indentical to tel1_ for opperational convenience when merged with trade data */ 
   area_code_tel2_=tel1_;  /* define this variable indentical to tel1_ for opperational convenience when merged with trade data */ 
run;



*===============;
*Read Panel Data;
*===============;
%include "F:\china project\Panel9807SAS\firm1999_update.sas"/source2;
libname firm1998 xport "F:\china project\Panel9807SAS\firm1998.xpt";
libname firm1999 xport "F:\china project\Panel9807SAS\firm1999.xpt";
libname firm2000 xport "F:\china project\Panel9807SAS\firm2000.xpt";
libname firm2001 xport "F:\china project\Panel9807SAS\firm2001.xpt";
libname firm2002 xport "F:\china project\Panel9807SAS\firm2002.xpt";
libname firm2003 xport "F:\china project\Panel9807SAS\firm2003.xpt";
libname firm2004 xport "F:\china project\Panel9807SAS\firm2004.xpt";
libname firm2005 xport "F:\china project\Panel9807SAS\firm2005.xpt";
libname firm2006 xport "F:\china project\Panel9807SAS\firm2006.xpt";
libname firm2007 xport "F:\china project\Panel9807SAS\firm2007.xpt";

proc copy in=firm1998 out=work; run;
proc copy in=firm1999 out=work; run;
proc copy in=firm2000 out=work; run;
proc copy in=firm2001 out=work; run;
proc copy in=firm2002 out=work; run;
proc copy in=firm2003 out=work; run;
proc copy in=firm2004 out=work; run;
proc copy in=firm2005 out=work; run;
proc copy in=firm2006 out=work; run;
proc copy in=firm2007 out=work; run;

%macro variables_compare(start=, end=);
data panel_tmp1;
run;

%do i=&start %to &end;
data firm&i._tmp1;
     set firm&i;
	 length legal_co_new $20.
	        legal_na_new $20.
			name_new     $62.
			phone_new    $20.  /* notice no phone number for 2006 */
			postcode_new $6.;
	 legal_co_new=legal_co;
	 legal_na_new=legal_na;
	 name_new    =name    ;
	 phone_new   =phone   ;
	 postcode_new=postcode;
	 keep legal_co_new legal_na_new name_new phone_new postcode_new year;
run;

data panel_tmp1;
    set panel_tmp1
	    firm&i._tmp1;
run;
%end;

%mend;
*%variables_compare(start=1998, end=2007);  *results given to Geying in Wuhan used matches with 1998-2007 data; 
%variables_compare(start=2000, end=2006);
*================================================================================================;
*Note: (1) For 2006, there is no phone number? (2) There are 795502 obs in frdm and company pairs;
*and there are 139031 frdm codes among them one frdm corresponds to multiple companies either    ;
*because (a) company names are spelled differently to represent same company (and often it can be;
*inferred from other information like pperson and phone number that it is the same company with  ;
*the other non empty company name. (3) Also note frdm_panel has length of 20 (actually           ;
*max(length)=10, not consistent with frdm defined in census data yet (which is 9), needs further :
*investigation?                                                                                  ;
*=================================================================================================;
data panel_id_tmp2;
   set panel_tmp1;
   length zip_pperson_ $30.
          company     $62.;
   company_tmp   =compress(name_new);
   if substr(company_tmp,1,1) in ('*', '.') then company=substr(company_tmp,2);
      else company =company_tmp;
   pperson       =compress(legal_na_new);  
   frdm_panel    =upcase(compress(legal_co_new));  
   tel1_tmp1     =0;
   tel1_tmp2     =compress(phone_new);
   if substr(tel1_tmp2,1,1)='0'   then tel1_tmp1=1;   
   if substr(tel1_tmp2,1,2)='00'  then tel1_tmp1=2;
   if substr(tel1_tmp2,1,3)='000' then tel1_tmp1=3;
   if tel1_tmp1=0 then tel1_=tel1_tmp2;
   if tel1_tmp1=1 then tel1_=substr(tel1_tmp2,2);
   if tel1_tmp1=2 then tel1_=substr(tel1_tmp2,3);
   if tel1_tmp1=3 then tel1_=substr(tel1_tmp2,4);
   zip_pperson_=compress(postcode_new||pperson);
   if (company='' and pperson='' and frdm_panel='' and tel1_='' and postcode_new='' ) then delete;
   keep frdm_panel company tel1_ zip_pperson_ year;
run;

*==================================================================================================;
*(1) There are duplicate obs by frdm_panel company, but no duplicate obs by frdm_panel company year;
*    so the duplicate frdm_panel company is due to obs from different year. We can get rid of those;
*    duplicated obs.                                                                               ; 
*(2) After I get rid of obs with duplicated frdm_panel company, there are still cases when one     ;
*    frdm_panel corresponds to multiple companies because company names are spelled differently to ;
*    represent same company. This is due to company names specified differently in different years.;                                                              ;
*==================================================================================================;
proc sort data=panel_id_tmp2 out=panel_id_tmp3 nodupkey; by frdm_panel company; run;

*================================================;
*Treat varialbes with irregular values as missing;
*================================================;
data panel_id_tmp4;
   set panel_id_tmp3;
   if substr(tel1_,1,1) in ('*','-','.') then tel1_=substr(tel1_,2); 
   if substr(tel1_,1,1) ='0'   or
      length(tel1_)      <5    or
      find(tel1_, 'E+10')    or
	  find(tel1_, 'E+11')    or
      find(tel1_, '10-')     or
      tel1_='?T1??'        or
      substr(upcase(tel1_),1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345','XXXXX','*****') then tel1_='';
   if substr(upcase(tel1_),1,1) in ('A','B','E','S','X',']') then tel1_=substr(tel1_,2); 
   if find(tel1_,'??')  or find(tel1_,'??D') or find(tel1_, '??90598') or find(tel1_, '???15-0') then tel1_='';	

   if substr(zip_pperson_,1,1)='0' then zip_pperson_=''; 
   if substr(zip_pperson_,1,1) ^in ('1','2','3','4','5','6','7','8','9') then zip_pperson_='';
   if length(zip_pperson_)<=6 then zip_pperson_='';
    if substr(company,1,1) in ('','"','-','.','0','[',']','`','h','i','I','n','r','s','t','y') then company=substr(company,2);
   if substr(company,1,1) in ('1','2','3','4','5','6','7','8','9') then company='';
   if substr(company,1,2) in ('??','?e','??','??,'??','??) then company=substr(company,3);
run;




data panel_id (rename=(company         =company_p
                        zip_pperson_   =zip_pperson_p
                        tel1_          =tel1_p

                        area_code_tel1_=area_code_tel1_p
                        tel2_          =tel2_p
                        area_code_tel2_=area_code_tel2_p
                        year           =year_p));
   set panel_id_tmp4;
   area_code_tel1_=tel1_;  /* define this variable indentical to tel1_ for opperational convenience when merged with trade data */ 
   tel2_          =tel1_;  /* define this variable indentical to tel1_ for opperational convenience when merged with trade data */ 
   area_code_tel2_=tel1_;  /* define this variable indentical to tel1_ for opperational convenience when merged with trade data */ 
run;


*==================================================================;
*In panel_id and foreign_id datasets, there maybe duplicated values;
*for each comparing variable company, zip_pperson, tel1, etc.I set ;
*those with more than &count_max number of occurance as missing '' ; 
*==================================================================;
%let count_max=120;
%include "F:\china project\macro code\count_duplicate_key.sas"/source2;
%count_duplicate_key(data_input=panel_id, data_output=panel_id, sorting_key=company_p       , count_max=&count_max);
%count_duplicate_key(data_input=panel_id, data_output=panel_id, sorting_key=zip_pperson_p   , count_max=&count_max);
%count_duplicate_key(data_input=panel_id, data_output=panel_id, sorting_key=tel1_p          , count_max=&count_max);
%count_duplicate_key(data_input=panel_id, data_output=panel_id, sorting_key=tel2_p          , count_max=&count_max);
%count_duplicate_key(data_input=panel_id, data_output=panel_id, sorting_key=area_code_tel1_p, count_max=&count_max);
%count_duplicate_key(data_input=panel_id, data_output=panel_id, sorting_key=area_code_tel2_p, count_max=&count_max);


%count_duplicate_key(data_input=foreign_id, data_output=foreign_id, sorting_key=company_f       , count_max=&count_max);
%count_duplicate_key(data_input=foreign_id, data_output=foreign_id, sorting_key=zip_pperson_f   , count_max=&count_max);
%count_duplicate_key(data_input=foreign_id, data_output=foreign_id, sorting_key=tel1_f          , count_max=&count_max);
%count_duplicate_key(data_input=foreign_id, data_output=foreign_id, sorting_key=tel2_f          , count_max=&count_max);
%count_duplicate_key(data_input=foreign_id, data_output=foreign_id, sorting_key=area_code_tel1_f, count_max=&count_max);
%count_duplicate_key(data_input=foreign_id, data_output=foreign_id, sorting_key=area_code_tel2_f, count_max=&count_max);

proc datasets lib=work;
   save panel_id foreign_id foreign;
run;

*=================================;
*Match panel data with foreign data;
*=================================;
%macro panel_foreign_matched(matching_key=);
data panel_id_tmp1;
     set panel_id0;
	 if &matching_key._p^='' then output;
run;

data foreign_id_tmp1;
   set foreign_id0;
   if &matching_key._f^='' then output; run;
run;

proc sql;
   create table panel_id_foreign_id_tmp1  as
   select a.*,
          b.*
   from panel_id_tmp1 as a, 
        foreign_id_tmp1 as b
   where a.&matching_key._p=b.&matching_key._f;
quit;

proc sort data=panel_id_foreign_id_tmp1 out=tmp1; by &matching_key._p;  run;

data tmp2 (keep=&matching_key._p count);
   set tmp1;
   by &matching_key._p;
   retain count 0;
   if first.&matching_key._p then count=0;
   count+1;
   if last.&matching_key._p then output;
run;

proc sort data=tmp2; by &matching_key._p; run;

data tmp3;
   merge tmp1
         tmp2;
   by &matching_key._p;
run;

data panel_id_foreign_id_tmp2;
   set tmp3;
   array var_p[*] company_p  zip_pperson_p  tel1_p tel2_p area_code_tel1_p area_code_tel2_p; 
   array var_f[*] company_f  zip_pperson_f  tel1_f tel2_f area_code_tel1_f area_code_tel2_f;
   variables_matched=0;
   do i=1 to dim(var_p);
      if (var_p[i]^='' and var_p[i]=var_f[i]) then variables_matched+1;
   end;
   drop i;
run;

data panel_id_foreign_id_tmp3  (keep =foreign_id frdm_panel);
    set panel_id_foreign_id_tmp2;
	if (count=1) or (count>1 & variables_matched>1) then output;
run;

proc sort data=panel_id_foreign_id_tmp3                                      nodupkey; by frdm_panel foreign_id ; run;
proc sort data=panel_id_foreign_id_tmp3 out=panel_id_found (keep=frdm_panel)   nodupkey; by frdm_panel          ; run;
proc sort data=panel_id_foreign_id_tmp3 out=foreign_id_found (keep=foreign_id) nodupkey; by foreign_id       ; run;

proc sort data=panel_id0; by frdm_panel  ; run;
proc sort data=foreign_id0; by foreign_id; run; 

data panel_id0;
    merge panel_id0      (in=a)
	      panel_id_found (in=b);
    by frdm_panel;
    if (a=1 & b^=1) then output;
run;

data foreign_id0;
    merge foreign_id0       (in=a)
          foreign_id_found  (in=b);
	by foreign_id;
	if (a=1 & b^=1) then output;
run;

data panel_id_foreign_id;
   set panel_id_foreign_id
       panel_id_foreign_id_tmp3;
run;
 
proc datasets lib=work;
   delete tmp1 - tmp3
          panel_id_tmp1 
		  panel_id_found
          foreign_id_tmp1
          foreign_id_found 
          panel_id_foreign_id_tmp1
          panel_id_foreign_id_tmp2
          panel_id_foreign_id_tmp3;
run;
%mend;

%macro loop_panel_foreign_matched(matching_key_lst_n =,
                                 matching_key_lst   =);
     %do i=1 %to &matching_key_lst_n;
         %let matching_key =%scan(&matching_key_lst, &i);
	     %panel_foreign_matched(matching_key=&matching_key);
     %end;
%mend;


data panel_id0 (drop=year_p); set panel_id; run;
data foreign_id0; set foreign_id; run;
data panel_id_foreign_id    ;               run;

%loop_panel_foreign_matched(matching_key_lst_n=6,
                             matching_key_lst=company zip_pperson  tel1 tel2 area_code_tel1 area_code_tel2);

data panel_id_foreign_id; set panel_id_foreign_id; if _n_^=1 then output; run;


proc sql;
  create table panel_id_foreign_id_tmp1 as
  select a.*,
         b.*
  from panel_id as a,
       foreign_id as b,
	   panel_id_foreign_id as c
  where a.frdm_panel=c.frdm_panel and
        b.foreign_id=c.foreign_id;
quit;


data panel_id_foreign_id_tmp2;
   set panel_id_foreign_id_tmp1;
   array var_p[*]     tel1_p   tel2_p   area_code_tel1_p   area_code_tel2_p; 
   array var_f[*]     tel1_f   tel2_f   area_code_tel1_f   area_code_tel2_f;
   array var_ind[*]   tel1_ind tel2_ind area_code_tel1_ind area_code_tel2_ind;
   matched_weight    =0;
   company_ind       =0;
   zip_pperson_ind   =0;
   zip_or_pperson_ind=0;
   if company_p^=''       and company_p=company_f         then company_ind    =1.5;   matched_weight=matched_weight + company_ind   ;
   if zip_pperson_p^=''   and zip_pperson_p=zip_pperson_f then zip_pperson_ind=1.5;   matched_weight=matched_weight + zip_pperson_ind;

   if zip_pperson_p^='' then do;
      if (substr(zip_pperson_f,1,6)=substr(zip_pperson_p,1,6))        
          or (compress(substr(zip_pperson_f,7))=compress(substr(zip_pperson_p,7)))
              then zip_or_pperson_ind=0.5;
   end;

   matched_weight=matched_weight + zip_or_pperson_ind;

   do i=1 to dim(var_p);
      var_ind[i]=0;
      if (var_p[i]^='' and var_p[i]=var_f[i]) then var_ind[i]=1;
      matched_weight=matched_weight+var_ind[i];
   end;  
   drop  i;
run;

data panel_id_foreign_id_stat;
   set panel_id_foreign_id_tmp2;
   if matched_weight>=1.5 then output;
run;

proc sort data=panel_id_foreign_id_stat
           out=panel_id_foreign_id   (keep=frdm_panel foreign_id rename=(frdm_panel=frdm_panel_new))  nodupkey; 
by frdm_panel foreign_id; 
run; 


*=========================================================;
*link foreign dataset with panel using subject constraints;
*=========================================================;
data foreign (rename=(couname1=country_f industry_code=industry_code_f labor=labor_f flabor=flabor_f age=age_f area_code=area_code_f));
   set foreign;
   keep foreign_id  couname1 industry_code  labor flabor age  area_code;
run;

proc sql;
  create table foreign_tmp1 as
  select a.*,
         c.frdm_panel_new,
		 c.ownership,
		 c.year,
		 c.area_code, 
		 c.cic_adj
  from foreign as a, 
       panel_id_foreign_id as b,
       output.firm_characteristics as c
 where a.foreign_id=b.foreign_id         and
       c.frdm_panel_new=b.frdm_panel_new and
       c.year>=2000                     and
       c.year<=2006;
quit;  

data foreign_tmp2;
   set foreign_tmp1;
   length country $33.;   
   if industry_code_f^=. then industry_code_f_2dgts=substr(compress(put(industry_code_f,4.)),1,2);
   if cic_adj^=.         then cic_adj_2dgts        =substr(compress(put(cic_adj,4.)),1,2);
   if area_code_f^=.     then area_code_f_2dgts    =substr(compress(put(area_code_f,6.)),1,2);
   if area_code^=.       then area_code_2dgts      =substr(compress(put(area_code,6.)),1,2);
   country=country_f;
   if country="???????" then country="????a???";
   if country="?a??" then country="??2?ao???3??;
   if country="??'? then country="??'oo";
   if country="??T????1?? then country="";
   if country="??T?1" then country="??T?1a?";
   if country="1t???" then country="1t????1?";
   if country="?Y??" then country="?Y??12o??;
   if country="?-?oo" then country="?a?oo";
   if country="???oo" then country="???oo12o??;
   if country="???1?" then country="???1?a??12o??;
   if country="??T????1?? then country="";
   if country="???????o" then country="";
   if country="??" then country="????2?";
   if country="?1?????" then country="?1?????12o??;
   if country="?????" then country="?????o???";
   if country="??????oo" then country="????????oo";
   if country="??? then country="??????";
   if country="?D1? then country="?D?a???12o??;
   if country="???|??" then country="??|??";
   if industry_code_f_2dgts=cic_adj_2dgts & area_code_f_2dgts=area_code_2dgts then output;
   keep year frdm_panel_new country labor_f flabor_f; 
run;

proc import datafile="F:\china project\exchange_rate_calculation\UN_country_code_11_5.csv"
                     out=isocode  replace;
run;
data isocode;  set isocode; keep country isocode; run;
data isocode; set isocode; if isocode="TAI" then isocode="TWN"; run;
proc sort data=isocode nodupkey; by country; run;
proc sort data=foreign_tmp2; by country; run;

data foreign_tmp3 (drop=country rename=(isocode=isocode_f));
   merge foreign_tmp2 (in=a)
         isocode (in=b);
   by country;
   if a & b then output;
run;


data gdpcap_fie (rename=(isocode=isocode_f gdpcap=gdpcap_f));
  set output.gravity;
  keep year isocode gdpcap;
  if year>=2000 and year<=2006 and gdpcap^=. then output;
run;

data gdpcap_chn (rename=(gdpcap=gdpcap_chn));
   set output.gravity;
   if isocode="CHN" and year>=2000 and year<=2006 then output;
   keep year gdpcap;
run;




proc sort data=gdpcap_fie nodupkey; by year isocode_f;  run;
proc sort data=foreign_tmp3; by year isocode_f; run;

data foreign_tmp3;
   merge foreign_tmp3 (in=a)
         gdpcap_fie   (in=b);
   by year isocode_f;
   if a and b then output;
run;

*=================================================================;
*identify 472 pairs of duplicates frdm_panel_new year, delete them;
*=================================================================;
proc sort data=foreign_tmp3 out=foreign_tmp30; by year frdm_panel_new; run;

data foreign_tmp31;
   set foreign_tmp30;
   by year frdm_panel_new;
   retain count;
   if first.frdm_panel_new then count=0;
   count+1;
   if last.frdm_panel_new and count>1 then output;
   keep frdm_panel_new year;
run;

data foreign_tmp4;
   merge foreign_tmp30 (in=a) 
         foreign_tmp31 (in=b);
   by year frdm_panel_new;
   if a=1 and b=0 then output;
run;

proc sort data=foreign_tmp4; by frdm_panel_new year; run;
proc sort data=export.reg_data_export_by_product_
           out =reg_data_export_by_product_ 
                (keep=year 
                      frdm_panel_new 
                      value 
                      valperunit_mean 
                      hs96_01  
                      isocode 
                      ownership
					  l_sale
					  lag_l_k
					  lag_l_labor
					  lag_l_wage
					  lag_tfp_lp_value_added
					  gdp
					  gdpcap
 				  remoteness)
           tagsort; 
   by year; 
run;

proc sort data=gdpcap_chn; by year; run;

data reg_data_export_by_product_;
   merge reg_data_export_by_product_ (in=a)
         gdpcap_chn                  (in=b);
   by year;
   if a & b then output;
run;

proc sort data=reg_data_export_by_product_ tagsort; by frdm_panel_new year; run;
proc sort data=foreign_tmp4                       ; by frdm_panel_new year; run;

data export.reg_data_fie (drop=gdpcap_chn);
    merge reg_data_export_by_product_ (in=a)
	      foreign_tmp4                (in=b);
	by frdm_panel_new year;
    fie_ind      =0;
    fie_match_ind=0;
	if ownership="DOM" then do;
       isocode_f="CHN";
	   gdpcap_f=gdpcap_chn;
	end;
    if ownership="FIE" then fie_ind=1;
    if flabor_f^=. and ownership="FIE" and  isocode_f ^in ('HKG' 'MAC' 'TWN') then fie_match_ind=1;
	if a then output;
run;




