*===========================================================;
*This code is updated from matching_trade_census_update6.sas;
*It creates matching ids between trade and census, between  ;
*panel and census, and between trade and panel data         ;
*Notice                                                     ;
*Aug10 data given to Wuhan meeting used matched with 1998-  ;
*2007 panel data, now revise using 2000 - 2006 panel, where ;
*2006 panel data has been updated by adding phone number    ; 
*===========================================================;
libname b603 xport "D:\china project\Census04SAS\b603.xpt";
libname b604 xport "D:\china project\Census04SAS\b604.xpt";
libname census10 xport "D:\china project\Census04SAS\census2004b610.xpt";
libname censusgs xport "D:\china project\Census04SAS\census2004gs601.xpt";
libname censusgx "D:\china project\Census04SAS";
libname trade_fm "D:\china project\Tradefirm_SAS";
libname output "D:\china project\sas_output_data";

*=====================;
*Read Census 2004 data;
*=====================;
proc copy in=b603     out=work; run;
proc copy in=b604     out=work; run;
proc copy in=census10 out=work; run;
proc datasets lib=work; change census20=census2004b610; run;
proc copy in=censusgs out=work; run;
proc datasets lib=work; change census20=census2004gs601;run;

data census2004gx601;
   set censusgx.census2004gx601;
run;

data census2004_tmp1;
   set census2004gs601
       census2004gx601;
   keep frdm b02 b031 b060 b061 b063 b065 b066;
run;

*=================================================================================================;
*Define and form matching keys for the census data. Notice: (1) there are only 322 dupkey on frdm.;
*Visual examination shows the related pairs are identical from each other so can delete redundant.;
*(2) there is 1-1 correspondence between a frdm and b02, or company. (3) b063, or fax number are  ;
*all less than 8 char (4) b061, or tel, has 788 observations with 9 & 10 digits which is irregular;
*(5)b061, or tel, has 114837 observations with 11 digits, these are cell phone number             ;
*=================================================================================================;
proc sort data=census2004_tmp1 out=census2004_tmp2 nodupkey; by frdm;  run;

data census2004_tmp3;
   set census2004_tmp2;
   length company        $62. 
          zip_pperson    $30.
          e_mail         $40.
		  tel1           $20. 
          area_code_tel1 $20.
		  fax1           $20.
		  area_code_fax1 $20.
          area_code_tel1_tmp1 $20.;
 
   company=compress(b02);
   zip    =compress(b065);
   pperson=compress(b031);
   e_mail =compress(b066);
   tel1   =compress(b061);
   len_tel1=length(compress(tel1));
   if len_tel1<5 then area_code_tel1_tmp1='';
   else if len_tel1<11 then area_code_tel1_tmp1=compress(b060||b061);
	  else area_code_tel1_tmp1=compress(b061); 
   area_code_tel1_tmp2=0;
   if length(area_code_tel1_tmp1)>1 then do;
      if substr(area_code_tel1_tmp1,1,1)='0'   then area_code_tel1_tmp2=1;
      if substr(area_code_tel1_tmp1,1,2)='00'  then area_code_tel1_tmp2=2;
      if substr(area_code_tel1_tmp1,1,3)='000' then area_code_tel1_tmp2=3;
      if area_code_tel1_tmp2=0 then area_code_tel1=area_code_tel1_tmp1;
      if area_code_tel1_tmp2=1 then area_code_tel1=substr(area_code_tel1_tmp1,2);
      if area_code_tel1_tmp2=2 then area_code_tel1=substr(area_code_tel1_tmp1,3);
      if area_code_tel1_tmp2=3 then area_code_tel1=substr(area_code_tel1_tmp1,4);
   end;

   fax1   =compress(b063);
   if fax1^='' then area_code_fax1_tmp1=compress(b060||b063);
      else          area_code_fax1_tmp1=''                  ; 
   area_code_fax1_tmp2=0;
   if length(area_code_fax1_tmp1)>1 then do;
      if substr(area_code_fax1_tmp1,1,1)='0'   then area_code_fax1_tmp2=1;
      if substr(area_code_fax1_tmp1,1,2)='00'  then area_code_fax1_tmp2=2;
      if substr(area_code_fax1_tmp1,1,3)='000' then area_code_fax1_tmp2=3;
      if area_code_fax1_tmp2=0 then area_code_fax1=area_code_fax1_tmp1;
      if area_code_fax1_tmp2=1 then area_code_fax1=substr(area_code_fax1_tmp1,2);
      if area_code_fax1_tmp2=2 then area_code_fax1=substr(area_code_fax1_tmp1,3);
      if area_code_fax1_tmp2=3 then area_code_fax1=substr(area_code_fax1_tmp1,4);
	end;
   tel2=tel1; area_code_tel2=area_code_tel1;   /* notice these are identical pair for futher programming convenience if matching step */
   fax2=fax1; area_code_fax2=area_code_fax1;   /* notice these are identical pair for futher programming convenience if matching step */
   zip_pperson=compress(zip||pperson);
   frdm_census2004=upcase(compress(frdm));
   keep frdm_census2004 company    e_mail zip_pperson  
        tel1 area_code_tel1 tel2   area_code_tel2
        fax1 area_code_fax1 fax2   area_code_fax2;
run;

data census2004_id (rename=(company      =company_c
                            e_mail        =e_mail_c 
                            zip_pperson   =zip_pperson_c
                            tel1          =tel1_c
                            area_code_tel1=area_code_tel1_c
                            tel2          =tel2_c
                            area_code_tel2=area_code_tel2_c
                            fax1          =fax1_c
                            area_code_fax1=area_code_fax1_c
                            fax2          =fax2_c
                            area_code_fax2=area_code_fax2_c));
    set census2004_tmp3;
	if substr(company,1,1)='.' then company=substr(company,2);
	do i=1 to 13;
	   if substr(company,1,1) in ('0','1','2','3','4','5','6','7','8','9',';','*') then company=substr(company,2);                           
	end;
	drop i;
	if find(e_mail,'@')=0 then e_mail='';
	e_mail=upcase(e_mail);
    if substr(zip_pperson,1,1)='0' then zip_pperson=''; 
	if length(zip_pperson)<6       then zip_pperson='';
    if substr(tel1,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then tel1='';
	if substr(area_code_tel1,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_tel1='';
    if substr(area_code_tel1,4,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_tel1=''; 

    if substr(tel2,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then tel2='';
	if substr(area_code_tel2,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_tel2='';
    if substr(area_code_tel2,4,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_tel2=''; 

    if substr(fax1,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then fax1='';
	if substr(area_code_fax1,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_fax1='';
    if substr(area_code_fax1,4,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_fax1=''; 

    if substr(fax2,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then fax2='';
	if substr(area_code_fax2,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_fax2='';
    if substr(area_code_fax2,4,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_fax2=''; 
run;

*=======================;
*Read Import-Export Data;
*=======================;
%macro export_all(year=);
libname lib&year "D:\china project\SAS_Trade_Data\&year";
data data&year; run;
%do i=1 %to 12;
data data&year.&i;
   length company1 $62.;
   %if %eval(&i)<10 %then %do;
      set lib&year..data&year.0&i;
   %end;
   %else %do;
      set lib&year..data&year.&i;
   %end;
   %if %eval(&year)<2004 %then %do;
        e_mail=email;
   %end;
   %if %eval(&year)=2005 and %eval(&i)=12 %then %do;
        e_mail=email;
   %end;
   year=&year;
   month=&i;
   company1=company;
   keep party_id company1 tel zip fax e_mail pperson year month; 
run;
proc sort data=data&year.&i nodupkey; by party_id company1; run;
data data&year;
   set data&year
       data&year.&i;
run;
%end;
proc sort data=data&year nodupkey; by party_id company1 year; run;
%mend;

%export_all(year=2000);
%export_all(year=2001);
%export_all(year=2002);
%export_all(year=2003);
%export_all(year=2004);
%export_all(year=2005);
%export_all(year=2006);

data trade_tmp1 (rename=(company1=company));
    set
	    data2000
	    data2001
	    data2002
		data2003 
		data2004
		data2005
		data2006;
		if party_id^='' then output; 
run;

proc sort data=trade_tmp1 out=trade_tmp2 nodupkey; by party_id company; run; 

*==============================================================================================;
*(1) There are duplicate obs by party_id company, but no duplicate obs by party_id company year;
*    so the duplicate party_id company is due to obs from different years. We can get rid of   ;
*    those duplicated obs.                                                                     ; 
*(2) After I get rid of obs with duplicated party_id company, there are still cases when one   ;
*    party_id corresponds to multiple companies because company names are spelled differently  ;
*    to represent same company. This is due to company names specified differently in different;
*    years. in some cases one company name is empty but can be inferred from other information ;
*    such as pperson and phone number that it is the same company                              ;
*==============================================================================================;
data  trade_tmp3 trade_tmp4;
    set trade_tmp2;
	if (company='' and tel='' and fax='' and zip='' and pperson='' and e_mail='') then output trade_tmp3;
	else output trade_tmp4;
	*get rid of those with no information to match;
run;

data trade;
   set trade_tmp4;
   if length(e_mail) in (5 6) then do;
      zip=e_mail;
	  e_mail='';
   end;
   *====================================================================;
   *2006 trade files have some e_mail values wrongly coded with zip code;
   *and corresponding zip code column empty. Assign those e_mail to zip ;
   *====================================================================; 
run;


data trade_id_tmp1;
    set trade;
    length tel1_           $20.
           area_code_tel1_ $20.
           tel2_           $20.
           area_code_tel2_ $20.
           fax1_           $20. 
           area_code_fax1_ $20.
           fax2_           $20.
           area_code_fax2_ $20.
           zip_pperson_    $30.;

    tel_part1=scan(tel,1,' -/()'); len_tel_part1=length(tel_part1);
	tel_part2=scan(tel,2,' -/()'); len_tel_part2=length(tel_part2);
	tel_part3=scan(tel,3,' -/()'); len_tel_part3=length(tel_part3);
	
	if len_tel_part1<=5 then do;
        if len_tel_part2=1 then tel1_=tel_part1;
		   else area_code=tel_part1;
	   end;
	   else if len_tel_part1<=8 then tel1_=tel_part1;
	       else area_code_tel_=tel_part1;

	if len_tel_part2>=5 then do;
	   if length(tel1_)=1 then tel1_=tel_part2;
	      else tel2_=tel_part2; 
	end;
 
	if len_tel_part3>=5 then do;
       if length(tel2_)=1 then tel2_=tel_part3;
	end;

   if area_code_tel_='' then area_code_tel_=compress(area_code||tel1_);
   tel_ind=-1;
   if length(area_code)>1 and substr(area_code_tel_,1,1)^='0' then tel_ind=0;
   if len_tel_part1>9                                         then tel_ind=0;
   if substr(area_code_tel_,1,1)='0'   then tel_ind =1;
   if substr(area_code_tel_,1,2)='00'  then tel_ind =2;
   if substr(area_code_tel_,1,3)='000' then tel_ind =3;
   if tel_ind=0 then area_code_tel1_=area_code_tel_          ;
   if tel_ind=1 then area_code_tel1_=substr(area_code_tel_,2);
   if tel_ind=2 then area_code_tel1_=substr(area_code_tel_,3);
   if tel_ind=3 then area_code_tel1_=substr(area_code_tel_,4);

   if tel2_^='' and area_code^='' then do;
      tel2_tmp1=compress(area_code||tel2_);
	  tel2_tmp2=0;
	  if substr(tel2_tmp1,1,1)='0'   then tel2_tmp2=1;
	  if substr(tel2_tmp1,1,2)='00'  then tel2_tmp2=2;
	  if substr(tel2_tmp1,1,3)='000' then tel2_tmp2=3;
      if tel2_tmp2=0 then area_code_tel2_=tel2_tmp1;
	  if tel2_tmp2=1 then area_code_tel2_=substr(tel2_tmp1,2);
	  if tel2_tmp2=2 then area_code_tel2_=substr(tel2_tmp1,3);
	  if tel2_tmp2=3 then area_code_tel2_=substr(tel2_tmp1,4);
   end;
  
   fax_part1=scan(fax,1,' -/()'); len_fax_part1=length(fax_part1);
   fax_part2=scan(fax,2,' -/()'); len_fax_part2=length(fax_part2);
   fax_part3=scan(fax,3,' -/()'); len_fax_part3=length(fax_part3);

   if len_fax_part1<=5 then do;
       if len_fax_part2=1 then fax1_=fax_part1;
	      else area_code=fax_part1;
   end;
	   else if len_fax_part1<=8 then fax1_=fax_part1;
	      else area_code_fax1_=fax_part1;

   if len_fax_part2>=5 then do;
	  if length(fax1_)=1 then fax1_=fax_part2;
	     else fax2_=fax_part2; 
   end;
 
   if len_fax_part3>=5 then do;
      if length(fax2_)=1 then fax2_=fax_part3;
   end;
   if area_code_fax_='' then area_code_fax_=compress(area_code||fax1_);
   fax_ind=-1;
   if length(area_code)>1 and substr(area_code_fax_,1,1)^='0' then fax_ind=0;
   if len_fax_part1>9                                         then fax_ind=0;
   if length(area_code_fax_)>1 then do;
      if substr(area_code_fax_,1,1)='0'   then fax_ind=1;
      if substr(area_code_fax_,1,2)='00'  then fax_ind=2;
      if substr(area_code_fax_,1,3)='000' then fax_ind=3;
      if fax_ind=0 then area_code_fax1_=area_code_fax_          ;
      if fax_ind=1 then area_code_fax1_=substr(area_code_fax_,2);
      if fax_ind=2 then area_code_fax1_=substr(area_code_fax_,3);
      if fax_ind=3 then area_code_fax1_=substr(area_code_fax_,4);
	end;

   if fax2_^='' and area_code^='' then do;
      fax2_tmp1=compress(area_code||fax2_);
	  fax2_tmp2=0;
      if substr(fax2_tmp1,1,1)='0'   then fax2_tmp2=1;
	  if substr(fax2_tmp1,1,2)='00'  then fax2_tmp2=2;
	  if substr(fax2_tmp1,1,3)='000' then fax2_tmp2=3;
      if fax2_tmp2=0 then area_code_fax2_=fax2_tmp1;
	  if fax2_tmp2=1 then area_code_fax2_=substr(fax2_tmp1,2);
	  if fax2_tmp2=2 then area_code_fax2_=substr(fax2_tmp1,3);
	  if fax2_tmp2=3 then area_code_fax2_=substr(fax2_tmp1,4);
   end;
  zip_pperson_=compress(zip||pperson);
  e_mail_=e_mail;
  keep party_id   company         e_mail_  zip_pperson_     
        tel1_     area_code_tel1_ tel2_    area_code_tel2_ 
        fax1_     area_code_fax1_ fax2_    area_code_fax2_ 
        year                                           ;
run;


*================================================================================;
*this is actually repetition of proc sort data=trade_tmp1 out=trade_tmp2 nodupkey;
*should have no impact (no obs deletion) , served as a reminder here             ;
*================================================================================;
proc sort data=trade_id_tmp1 out=trade_id_tmp2 nodupkey; by party_id company; run;  

*================================================;
*Treat varialbes with irregular values as missing;
*================================================;
data trade_id_tmp3;
   set trade_id_tmp2;
   array var_[*] tel1_           tel2_           fax1_           fax2_   
                 area_code_tel1_ area_code_tel2_ area_code_fax1_ area_code_fax2_ 
                 e_mail_         zip_pperson_;
   do i=1 to dim(var_);
      if substr(var_[i],1,1)='0' or length(var_[i])<5 then var_[i]='';
   end;
   drop i;

   if length(compress(company))<5 then company=''; 
   if substr(company,1,1)='.' then company=substr(company,2);
   if substr(company,1,1) in ('0','1','2','3','4','5','6','7','8','9') then company='';

   if substr(tel1_,1,1) in ('+','.') then tel1_=substr(tel1_,2);
   if substr(tel1_,1,5)=('.....') then tel1_='';
   if substr(upcase(tel1_),1,3) in ('FAX', 'FEX')  then tel1_='';
   if substr(tel1_,1,2)='' then tel1_='';   

   if area_code_tel1_='' then area_code_tel1_='';
   if tel2_ in ('99]001','8251', '8273', '55730', '8880')  then tel2_='';
   if area_code_tel2_='100088*3709' then area_code_tel2_='';
   if substr(upcase(fax1_),1,5)='XXXXX' then fax1_='';
   if find(fax2_,'.') then fax2_='';
   if find(area_code_fax2_,',') then area_code_fax2_='';

   if find(zip_pperson_,'*') or find(zip_pperson_,'-') then zip_pperson_='';
   if zip_pperson_ ='.27620ʯ' then zip_pperson_='27620ʯ';
   if zip_pperson_ ='.51088' then zip_pperson_='51088';
   if zip_pperson_ ='/' then zip_pperson_='';
   if length(compress(zip_pperson_)) <=6 then zip_pperson_='';

   if substr(zip_pperson_,1,1)='0' then zip_pperson_=''; 
   if substr(zip_pperson_,1,1) ^in ('1','2','3','4','5','6','7','8','9') then zip_pperson_='';
   if find(e_mail_,'@')=0 then e_mail_='';
   e_mail_=upcase(e_mail_);   

   if substr(tel1_,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then tel1_='';
	if substr(area_code_tel1_,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_tel1_='';
    if substr(area_code_tel1_,4,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_tel1_=''; 

    if substr(tel2_,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then tel2_='';
	if substr(area_code_tel2_,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_tel2_='';
    if substr(area_code_tel2_,4,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_tel2_=''; 

    if substr(fax1_,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then fax1_='';
	if substr(area_code_fax1_,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_fax1_='';
    if substr(area_code_fax1_,4,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_fax1_=''; 

    if substr(fax2_,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then fax2_='';
	if substr(area_code_fax2_,1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_fax2_='';
    if substr(area_code_fax2_,4,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345') then area_code_fax2_=''; 
run;

data trade_id (rename=(company         =company_t
                        e_mail_        =e_mail_t 
                        zip_pperson_   =zip_pperson_t
                        tel1_          =tel1_t
                        area_code_tel1_=area_code_tel1_t
                        tel2_          =tel2_t
                        area_code_tel2_=area_code_tel2_t
                        fax1_          =fax1_t
                        area_code_fax1_=area_code_fax1_t
                        fax2_          =fax2_t
                        area_code_fax2_=area_code_fax2_t
                        year           =year_t));
   set trade_id_tmp3;
run;


*===============;
*Read Panel Data;
*===============;
%include "D:\china project\Panel9807SAS\firm1999_update.sas"/source2;
libname firm1998 xport "D:\china project\Panel9807SAS\firm1998.xpt";
libname firm1999 xport "D:\china project\Panel9807SAS\firm1999.xpt";
libname firm2000 xport "D:\china project\Panel9807SAS\firm2000.xpt";
libname firm2001 xport "D:\china project\Panel9807SAS\firm2001.xpt";
libname firm2002 xport "D:\china project\Panel9807SAS\firm2002.xpt";
libname firm2003 xport "D:\china project\Panel9807SAS\firm2003.xpt";
libname firm2004 xport "D:\china project\Panel9807SAS\firm2004.xpt";
libname firm2005 xport "D:\china project\Panel9807SAS\firm2005.xpt";
libname firm2006 xport "D:\china project\Panel9807SAS\firm2006.xpt";
libname firm2007 xport "D:\china project\Panel9807SAS\firm2007.xpt";

proc copy in=firm1998 out=work; run;
proc copy in=firm1999 out=work; run;
proc copy in=firm2000 out=work; run;
proc copy in=firm2001 out=work; run;
proc copy in=firm2002 out=work; run;
proc copy in=firm2003 out=work; run;
proc copy in=firm2004 out=work; run;
proc copy in=firm2005 out=work; run;
proc copy in=firm2006 out=work; run;
proc copy in=firm2007 out=work; run;

%macro variables_compare(start=, end=);
data panel_tmp1;
run;

%do i=&start %to &end;
data firm&i._tmp1;
     set firm&i;
	 length legal_co_new $20.
	        legal_na_new $20.
			name_new     $62.
			phone_new    $20.  /* notice no phone number for 2006 */
			postcode_new $6.;
	 legal_co_new=legal_co;
	 legal_na_new=legal_na;
	 name_new    =name    ;
	 phone_new   =phone   ;
	 postcode_new=postcode;
	 keep legal_co_new legal_na_new name_new phone_new postcode_new year;
run;

data panel_tmp1;
    set panel_tmp1
	    firm&i._tmp1;
run;
%end;

%mend;
*%variables_compare(start=1998, end=2007);  *results given to Geying in Wuhan used matches with 1998-2007 data; 
%variables_compare(start=2000, end=2006);
*================================================================================================;
*Note: (1) For 2006, there is no phone number? (2) There are 795502 obs in frdm and company pairs;
*and there are 139031 frdm codes among them one frdm corresponds to multiple companies either    ;
*because (a) company names are spelled differently to represent same company (and often it can be;
*inferred from other information like pperson and phone number that it is the same company with  ;
*the other non empty company name. (3) Also note frdm_panel has length of 20 (actually           ;
*max(length)=10, not consistent with frdm defined in census data yet (which is 9), needs further :
*investigation?                                                                                  ;
*=================================================================================================;
data panel_id_tmp2;
   set panel_tmp1;
   length zip_pperson_ $30.
          company     $62.;
   company_tmp   =compress(name_new);
   if substr(company_tmp,1,1) in ('*', '.') then company=substr(company_tmp,2);
      else company =company_tmp;
   pperson       =compress(legal_na_new);  
   frdm_panel    =upcase(compress(legal_co_new));  
   tel1_tmp1     =0;
   tel1_tmp2     =compress(phone_new);
   if substr(tel1_tmp2,1,1)='0'   then tel1_tmp1=1;   
   if substr(tel1_tmp2,1,2)='00'  then tel1_tmp1=2;
   if substr(tel1_tmp2,1,3)='000' then tel1_tmp1=3;
   if tel1_tmp1=0 then tel1_=tel1_tmp2;
   if tel1_tmp1=1 then tel1_=substr(tel1_tmp2,2);
   if tel1_tmp1=2 then tel1_=substr(tel1_tmp2,3);
   if tel1_tmp1=3 then tel1_=substr(tel1_tmp2,4);
   zip_pperson_=compress(postcode_new||pperson);
   if (company='' and pperson='' and frdm_panel='' and tel1_='' and postcode_new='' ) then delete;
   keep frdm_panel company tel1_ zip_pperson_ year;
run;

*==================================================================================================;
*(1) There are duplicate obs by frdm_panel company, but no duplicate obs by frdm_panel company year;
*    so the duplicate frdm_panel company is due to obs from different year. We can get rid of those;
*    duplicated obs.                                                                               ; 
*(2) After I get rid of obs with duplicated frdm_panel company, there are still cases when one     ;
*    frdm_panel corresponds to multiple companies because company names are spelled differently to ;
*    represent same company. This is due to company names specified differently in different years.;                                                              ;
*==================================================================================================;
proc sort data=panel_id_tmp2 out=panel_id_tmp3 nodupkey; by frdm_panel company; run;

*================================================;
*Treat varialbes with irregular values as missing;
*================================================;

data panel_id_tmp4;
   set panel_id_tmp3;
   if substr(tel1_,1,1) in ('*','-','.') then tel1_=substr(tel1_,2); 
   if substr(tel1_,1,1) ='0'   or
      length(tel1_)      <5    or
      find(tel1_, 'E+10')    or
	  find(tel1_, 'E+11')    or
      find(tel1_, '10-')     or
      tel1_='޹̶?'        or
      substr(upcase(tel1_),1,5) in ('10000', '11111','22222', '33333', '44444', '55555', '66666', '77777', '88888', '99999', '12345','XXXXX','*****') then tel1_='';
   if substr(upcase(tel1_),1,1) in ('A','B','E','S','X',']') then tel1_=substr(tel1_,2); 
   if find(tel1_,'')  or find(tel1_,'·') or find(tel1_, '590598') or find(tel1_, '15-0') then tel1_='';	

   if substr(zip_pperson_,1,1)='0' then zip_pperson_=''; 
   if substr(zip_pperson_,1,1) ^in ('1','2','3','4','5','6','7','8','9') then zip_pperson_='';
   if length(zip_pperson_)<=6 then zip_pperson_='';
    if substr(company,1,1) in ('','"','-','.','0','[',']','`','h','i','I','n','r','s','t','y') then company=substr(company,2);
   if substr(company,1,1) in ('1','2','3','4','5','6','7','8','9') then company='';
   if substr(company,1,2) in ('','','','','','') then company=substr(company,3);
run;

data panel_id (rename=(company         =company_p
                        zip_pperson_   =zip_pperson_p
                        tel1_          =tel1_p

                        area_code_tel1_=area_code_tel1_p
                        tel2_          =tel2_p
                        area_code_tel2_=area_code_tel2_p
                        year           =year_p));
   set panel_id_tmp4;
   area_code_tel1_=tel1_;  /* define this variable indentical to tel1_ for opperational convenience when merged with trade data */ 
   tel2_          =tel1_;  /* define this variable indentical to tel1_ for opperational convenience when merged with trade data */ 
   area_code_tel2_=tel1_;  /* define this variable indentical to tel1_ for opperational convenience when merged with trade data */ 
run;


*========================================================================;
*In census2004_id, trade_id, and panel_id datasets, there are duplicated ;
*values for each comparing variable company, zip_pperson, tel1, etc.I set;
*those with more than &count_max number of occurance as missing ''       ; 
*========================================================================;
%let count_max=120;
%include "D:\china project\macro code\count_duplicate_key.sas"/source2;
%count_duplicate_key(data_input=census2004_id, data_output=census2004_id, sorting_key=company_c       , count_max=&count_max);
%count_duplicate_key(data_input=census2004_id, data_output=census2004_id, sorting_key=zip_pperson_c   , count_max=&count_max);
%count_duplicate_key(data_input=census2004_id, data_output=census2004_id, sorting_key=e_mail_c        , count_max=&count_max);
%count_duplicate_key(data_input=census2004_id, data_output=census2004_id, sorting_key=tel1_c          , count_max=&count_max);
%count_duplicate_key(data_input=census2004_id, data_output=census2004_id, sorting_key=tel2_c          , count_max=&count_max);
%count_duplicate_key(data_input=census2004_id, data_output=census2004_id, sorting_key=fax1_c          , count_max=&count_max);
%count_duplicate_key(data_input=census2004_id, data_output=census2004_id, sorting_key=fax2_c          , count_max=&count_max);
%count_duplicate_key(data_input=census2004_id, data_output=census2004_id, sorting_key=area_code_tel1_c, count_max=&count_max);
%count_duplicate_key(data_input=census2004_id, data_output=census2004_id, sorting_key=area_code_tel2_c, count_max=&count_max);
%count_duplicate_key(data_input=census2004_id, data_output=census2004_id, sorting_key=area_code_fax1_c, count_max=&count_max);
%count_duplicate_key(data_input=census2004_id, data_output=census2004_id, sorting_key=area_code_fax2_c, count_max=&count_max);

%count_duplicate_key(data_input=trade_id, data_output=trade_id, sorting_key=company_t       , count_max=&count_max);
%count_duplicate_key(data_input=trade_id, data_output=trade_id, sorting_key=zip_pperson_t   , count_max=&count_max);
%count_duplicate_key(data_input=trade_id, data_output=trade_id, sorting_key=e_mail_t        , count_max=&count_max);
%count_duplicate_key(data_input=trade_id, data_output=trade_id, sorting_key=tel1_t          , count_max=&count_max);
%count_duplicate_key(data_input=trade_id, data_output=trade_id, sorting_key=tel2_t          , count_max=&count_max);
%count_duplicate_key(data_input=trade_id, data_output=trade_id, sorting_key=fax1_t          , count_max=&count_max);
%count_duplicate_key(data_input=trade_id, data_output=trade_id, sorting_key=fax2_t          , count_max=&count_max);
%count_duplicate_key(data_input=trade_id, data_output=trade_id, sorting_key=area_code_tel1_t, count_max=&count_max);
%count_duplicate_key(data_input=trade_id, data_output=trade_id, sorting_key=area_code_tel2_t, count_max=&count_max);
%count_duplicate_key(data_input=trade_id, data_output=trade_id, sorting_key=area_code_fax1_t, count_max=&count_max);
%count_duplicate_key(data_input=trade_id, data_output=trade_id, sorting_key=area_code_fax2_t, count_max=&count_max);

%count_duplicate_key(data_input=panel_id, data_output=panel_id, sorting_key=company_p       , count_max=&count_max);
%count_duplicate_key(data_input=panel_id, data_output=panel_id, sorting_key=zip_pperson_p   , count_max=&count_max);
%count_duplicate_key(data_input=panel_id, data_output=panel_id, sorting_key=tel1_p          , count_max=&count_max);
%count_duplicate_key(data_input=panel_id, data_output=panel_id, sorting_key=tel2_p          , count_max=&count_max);
%count_duplicate_key(data_input=panel_id, data_output=panel_id, sorting_key=area_code_tel1_p, count_max=&count_max);
%count_duplicate_key(data_input=panel_id, data_output=panel_id, sorting_key=area_code_tel2_p, count_max=&count_max);

proc datasets lib=work;
   save panel_id census2004_id trade_id;
run;

*======================================;
*Match trade data with census 2004 data;
*======================================;
%macro trade_census2004_matched(matching_key=);
data trade_id_tmp1;
     set trade_id0;
	 if &matching_key._t^='' then output;
run;

data census2004_id_tmp1;
   set census2004_id0;
   if &matching_key._c^='' then output; run;
run;

proc sql;
   create table trade_id_census2004_id_tmp1  as
   select a.*,
          b.*
   from trade_id_tmp1 as a, 
        census2004_id_tmp1 as b
   where a.&matching_key._t=b.&matching_key._c;
quit;

proc sort data=trade_id_census2004_id_tmp1 out=tmp1; by &matching_key._t;  run;

data tmp2 (keep=&matching_key._t count);
   set tmp1;
   by &matching_key._t;
   retain count 0;
   if first.&matching_key._t then count=0;
   count+1;
   if last.&matching_key._t then output;
run;

proc sort data=tmp2; by &matching_key._t; run;

data tmp3;
   merge tmp1
         tmp2;
   by &matching_key._t;
run;

data trade_id_census2004_id_tmp2;
   set tmp3;
   array var_t[*] company_t  zip_pperson_t  e_mail_t tel1_t tel2_t fax1_t fax2_t  area_code_tel1_t area_code_tel2_t area_code_fax1_t area_code_fax2_t;
   array var_c[*] company_c  zip_pperson_c  e_mail_c tel1_c tel2_c fax1_c fax2_c  area_code_tel1_c area_code_tel2_c area_code_fax1_c area_code_fax2_c;
   variables_matched=0;
   do i=1 to dim(var_t);
      if (var_t[i]^='' and var_t[i]=var_c[i]) then variables_matched+1;
   end;
   drop i;
run;

data trade_id_census2004_id_tmp3  (keep =frdm_census2004 party_id);
    set trade_id_census2004_id_tmp2;
	if (count=1) or (count>1 & variables_matched>1) then output;
run;

proc sort data=trade_id_census2004_id_tmp3                                                nodupkey; by party_id frdm_census2004 ; run;
proc sort data=trade_id_census2004_id_tmp3 out=trade_id_found (keep=party_id)             nodupkey; by party_id                 ; run;
proc sort data=trade_id_census2004_id_tmp3 out=census2004_id_found (keep=frdm_census2004) nodupkey; by frdm_census2004          ; run;

proc sort data=trade_id0     ; by party_id       ; run;
proc sort data=census2004_id0; by frdm_census2004; run; 

data trade_id0;
    merge trade_id0      (in=a)
	      trade_id_found (in=b);
    by party_id;
    if (a=1 & b^=1) then output;
run;

data census2004_id0;
    merge census2004_id0       (in=a)
          census2004_id_found  (in=b);
	by frdm_census2004;
	if (a=1 & b^=1) then output;
run;

data trade_id_census2004_id;
   set trade_id_census2004_id
       trade_id_census2004_id_tmp3;
run;
 

proc datasets lib=work;
   delete tmp1 - tmp3
          trade_id_tmp1 
		  trade_id_found
          census2004_id_tmp1
          census2004_id_found 
          trade_id_census2004_id_tmp1
          trade_id_census2004_id_tmp2
          trade_id_census2004_id_tmp3;
run;
%mend;


%macro loop_trade_census2004_matched(matching_key_lst_n =,
                                      matching_key_lst   =);
     %do i=1 %to &matching_key_lst_n;
         %let matching_key =%scan(&matching_key_lst, &i);
	     %trade_census2004_matched(matching_key=&matching_key);
     %end;
%mend;


data trade_id0  (drop=year); set trade_id     ; run;
data census2004_id0        ; set census2004_id; run;
data trade_id_census2004_id;                    run;

%loop_trade_census2004_matched(matching_key_lst_n=11,
                                   matching_key_lst  =company zip_pperson  e_mail tel1 tel2 fax1 fax2  area_code_tel1 area_code_tel2 area_code_fax1 area_code_fax2);

data trade_id_census2004_id; set trade_id_census2004_id; if _n_^=1 then output; run;

proc sql;
  create table trade_id_census2004_id_tmp1 as
  select a.*,
         b.*
  from trade_id as a,
       census2004_id as b,
	   trade_id_census2004_id as c
  where a.party_id       =c.party_id and
        b.frdm_census2004=c.frdm_census2004;
quit;

proc sort data=trade_id_census2004_id_tmp1; by party_id frdm_census2004; run;


data trade_id_census2004_id_tmp2;
   set trade_id_census2004_id_tmp1;
   array var_t[*]    e_mail_t   tel1_t   tel2_t   fax1_t   fax2_t    area_code_tel1_t   area_code_tel2_t   area_code_fax1_t   area_code_fax2_t  ;
   array var_c[*]    e_mail_c   tel1_c   tel2_c   fax1_c   fax2_c    area_code_tel1_c   area_code_tel2_c   area_code_fax1_c   area_code_fax2_c  ;
   array var_ind[*]  e_mail_ind tel1_ind tel2_ind fax1_ind fax2_ind  area_code_tel1_ind area_code_tel2_ind area_code_fax1_ind area_code_fax2_ind;
   matched_weight    =0;
   company_ind       =0;
   zip_pperson_ind   =0;
   zip_or_pperson_ind=0;
  
   if company_t^=''       and company_t=company_c         then company_ind    =1.5;   matched_weight=matched_weight + company_ind   ;
   if zip_pperson_t^=''   and zip_pperson_t=zip_pperson_c then zip_pperson_ind=1.5;   matched_weight=matched_weight + zip_pperson_ind;

   if zip_pperson_t^='' then do;
      if (substr(zip_pperson_c,1,6)=substr(zip_pperson_t,1,6))        
          or (compress(substr(zip_pperson_c,7))=compress(substr(zip_pperson_t,7)))
              then zip_or_pperson_ind=0.5;
   end;

   matched_weight=matched_weight + zip_or_pperson_ind;

   do i=1 to dim(var_t);
      var_ind[i]=0;
      if (var_t[i]^='' and var_t[i]=var_c[i]) then var_ind[i]=1;
      matched_weight=matched_weight+var_ind[i];
   end;
  
   drop  i;
run;

data trade_id_census2004_id_stat;
   set trade_id_census2004_id_tmp2;
   if matched_weight>=1.5 then output;
  * keep party_id frdm_census2004;
run;
 
proc sort data=trade_id_census2004_id_stat out=trade_id_census2004_id_update (keep= party_id frdm_census2004) nodupkey; by party_id frdm_census2004; run; 

proc datasets lib=work;
   delete trade_id_census2004_id_tmp1
          trade_id_census2004_id_tmp2;
run;

*================================;
*Match trade data with panel data;
*================================;
%macro trade_panel_matched(matching_key=);
data trade_id_tmp1;
     set trade_id0;
	 if &matching_key._t^='' then output;
run;

data panel_id_tmp1;
   set panel_id0;
   if &matching_key._p^='' then output; run;
run;

proc sql;
   create table trade_id_panel_id_tmp1  as
   select a.*,
          b.*
   from trade_id_tmp1 as a, 
        panel_id_tmp1 as b
   where a.&matching_key._t=b.&matching_key._p;
quit;

proc sort data=trade_id_panel_id_tmp1 out=tmp1; by &matching_key._t;  run;

data tmp2 (keep=&matching_key._t count);
   set tmp1;
   by &matching_key._t;
   retain count 0;
   if first.&matching_key._t then count=0;
   count+1;
   if last.&matching_key._t then output;
run;

proc sort data=tmp2; by &matching_key._t; run;

data tmp3;
   merge tmp1
         tmp2;
   by &matching_key._t;
run;

data trade_id_panel_id_tmp2;
   set tmp3;
   array var_t[*] company_t  zip_pperson_t  tel1_t tel2_t area_code_tel1_t area_code_tel2_t; 
   array var_p[*] company_p  zip_pperson_p  tel1_p tel2_p area_code_tel1_p area_code_tel2_p;
   variables_matched=0;
   do i=1 to dim(var_t);
      if (var_t[i]^='' and var_t[i]=var_p[i]) then variables_matched+1;
   end;
   drop i;
run;

data trade_id_panel_id_tmp3  (keep =frdm_panel party_id);
    set trade_id_panel_id_tmp2;
	if (count=1) or (count>1 & variables_matched>1) then output;
run;

proc sort data=trade_id_panel_id_tmp3                                      nodupkey; by party_id frdm_panel ; run;
proc sort data=trade_id_panel_id_tmp3 out=trade_id_found (keep=party_id)   nodupkey; by party_id            ; run;
proc sort data=trade_id_panel_id_tmp3 out=panel_id_found (keep=frdm_panel) nodupkey; by frdm_panel          ; run;

proc sort data=trade_id0; by party_id  ; run;
proc sort data=panel_id0; by frdm_panel; run; 

data trade_id0;
    merge trade_id0      (in=a)
	      trade_id_found (in=b);
    by party_id;
    if (a=1 & b^=1) then output;
run;

data panel_id0;
    merge panel_id0       (in=a)
          panel_id_found  (in=b);
	by frdm_panel;
	if (a=1 & b^=1) then output;
run;

data trade_id_panel_id;
   set trade_id_panel_id
       trade_id_panel_id_tmp3;
run;
 

proc datasets lib=work;
   delete tmp1 - tmp3
          trade_id_tmp1 
		  trade_id_found
          panel_id_tmp1
          panel_id_found 
          trade_id_panel_id_tmp1
          trade_id_panel_id_tmp2
          trade_id_panel_id_tmp3;
run;
%mend;

%macro loop_trade_panel_matched(matching_key_lst_n =,
                                 matching_key_lst   =);
     %do i=1 %to &matching_key_lst_n;
         %let matching_key =%scan(&matching_key_lst, &i);
	     %trade_panel_matched(matching_key=&matching_key);
     %end;
%mend;

data trade_id0 (drop=year); set trade_id; run;
data panel_id0 (drop=year); set panel_id; run;
data trade_id_panel_id    ;               run;

%loop_trade_panel_matched(matching_key_lst_n=6,
                             matching_key_lst=company zip_pperson  tel1 tel2 area_code_tel1 area_code_tel2);

data trade_id_panel_id; set trade_id_panel_id; if _n_^=1 then output; run;

proc sql;
  create table trade_id_panel_id_tmp1 as
  select a.*,
         b.*
  from trade_id as a,
       panel_id as b,
	   trade_id_panel_id as c
  where a.party_id  =c.party_id and
        b.frdm_panel=c.frdm_panel;
quit;

proc sort data=trade_id_panel_id_tmp1; by party_id frdm_panel; run;

data trade_id_panel_id_tmp2;
   set trade_id_panel_id_tmp1;
   array var_t[*]     tel1_t   tel2_t   area_code_tel1_t   area_code_tel2_t; 
   array var_p[*]     tel1_p   tel2_p   area_code_tel1_p   area_code_tel2_p;
   array var_ind[*]   tel1_ind tel2_ind area_code_tel1_ind area_code_tel2_ind;
   matched_weight    =0;
   company_ind       =0;
   zip_pperson_ind   =0;
   zip_or_pperson_ind=0;
   if company_t^=''       and company_t=company_p         then company_ind    =1.5;   matched_weight=matched_weight + company_ind   ;
   if zip_pperson_t^=''   and zip_pperson_t=zip_pperson_p then zip_pperson_ind=1.5;   matched_weight=matched_weight + zip_pperson_ind;

   if zip_pperson_t^='' then do;
      if (substr(zip_pperson_p,1,6)=substr(zip_pperson_t,1,6))        
          or (compress(substr(zip_pperson_p,7))=compress(substr(zip_pperson_t,7)))
              then zip_or_pperson_ind=0.5;
   end;

   matched_weight=matched_weight + zip_or_pperson_ind;

   do i=1 to dim(var_t);
      var_ind[i]=0;
      if (var_t[i]^='' and var_t[i]=var_p[i]) then var_ind[i]=1;
      matched_weight=matched_weight+var_ind[i];
   end;  
   drop  i;
run;

data trade_id_panel_id_stat;
   set trade_id_panel_id_tmp2;
   if matched_weight>=1.5 then output;
run;
 
proc sort data=trade_id_panel_id_stat out=trade_id_panel_id_update (keep=party_id frdm_panel) nodupkey; by party_id frdm_panel; run; 

proc datasets lib=work;
   delete trade_id_panel__id_tmp1
          trade_id_panel_id_tmp2;
run;

*================================;
*Match panel with census2004 data;
*================================;
proc sql;
   create table panel_id_census2004_id_tmp1  as
   select a.*,
          b.*
   from panel_id as a, 
        census2004_id as b
   where a.frdm_panel=b.frdm_census2004;
quit;

proc sort data=panel_id_census2004_id_tmp1 out=panel_id_census2004_id  nodupkey; by frdm_panel frdm_census2004; run;

data output.panel_id_census2004_id (keep=frdm_panel frdm_census2004) ;
    set panel_id_census2004_id;
run;

data output.trade_id_census2004_id_update;
    set trade_id_census2004_id_update;
run;

data output.trade_id_panel_id_update;
   set trade_id_panel_id;
run;

*===================================================================================;
*following data set to further calculate statistics to show what causes the matching;
*===================================================================================;

data output.trade_id_census2004_id_stat;
   set trade_id_census2004_id_stat;
run;

data output.trade_id_panel_id_stat;
   set trade_id_panel_id_stat;
run;


   



