*==============================================================================;
*This codes does:                                                              ;
*I0: Use Brandt et al. deflators and concordance to transform data             ;                                                               
*I   Calculate firm level TFP separately using real value added and real output;
*    for each of the two independent variables, do: (a),LP (b),OLS             ;                                                             ;
*II  Create two productivity measures used in Tybout (QJE 1998) & others       ;
*    (a): labor productivity, log(LAB)                                         ;
*    (b): average variable cost, log(AVC)                                      ;
*III  Create additional productivity measure used real value added per worker  ;
*!!Note: benefit variable missing for 2006 & 2007, the actual variable is      ;
*        total_benefit==>"if year=2005 or year=2007 then benefit=total_benefit"; 
*==============================================================================;
libname output "D:\china project\sas_output_data";
libname deflator "D:\china project\tfp_calculation\deflator";
libname census95 xport "D:\china project\Census95SAS\census1995.xpt";
libname transit  "D:\china project\export_import_decision_and_performance\transition";
%include "D:\china project\Panel9807SAS\firm1999_update.sas"/source2;

*===========================================================================================;
*I0: Read in deflators and industry concordance from Brandt et al. China websize            ;
*(1) output and input deflators: Yifan has there deflators for input and output respectively;
*    he suggests to use the benchmarks because the others are for robustness check purposes ;
*    input  deflator: benchmark_input_deflator.sas7bdat                                     ;
*    output deflator: benchmark_output_deflator.sas7bdat                                    ; 
*(2) Brandt and Rawski Investment deflator: brandt_rawski_capital_deflator                  ;
*(3) concordance: Firm classified into an industry following the 4-digit Chinese Industry   ;
*    Classification (CIC) that resembles the old US SIC system. To incorporate revision in  ;
*    2003, Brandt et al. developed a concordance:                                           ;
*    concordance:concordance_over_time.sdas7bdat                                            ;
*notice:  benchmark_input_deflator.dta has 424 observations while benchmark_output_deflator ;
*         has 423 observations. Both datasets share 422 cic_adj codes, cic_adj=2010 only    ;
*         appears in benchmark_output_deflator while cic_adj=2011 and 2012 only appear in   ;
*         benchmark_input_deflator. Sent email to Yifan for clarification.                  ;
*         a temporary solution: for benchmark_output_deflator, when cic_adj=2010 make it two;
*         rows, cic_adj=2011 and cic_adj=2012                                               ; 
*===========================================================================================;
%macro deflator_and_concordance(data=);
data &data.0;
   set deflator.&data;
run;
%mend;

%deflator_and_concordance(data=benchmark_input_deflator);
%deflator_and_concordance(data=benchmark_output_deflator);
%deflator_and_concordance(data=brandt_rawski_capital_deflator);
%deflator_and_concordance(data=concordance_over_time);


data benchmark_input_deflator;
   set benchmark_input_deflator0;
   array inputdefl_[10] inputdefl1998 - inputdefl2007;
   do i=1 to 10;
       inputdefl=inputdefl_[i];  
       year=1998+i-1;
	   output;
   end;
   keep year cic_adj inputdefl;
run;

*============================================================================;
*why output deflator has one less observation than input deflator? ask Yifan ;
*solution: when cic_adj=2010, make it two rows, cic_adj=2011 and cic_adj=2012;
*============================================================================;
data benchmark_output_deflator0_tmp1;
   set benchmark_output_deflator0;
   if cic_adj=2010 then do;
      cic_adj=2011; output;
	  cic_adj=2012; 
   end;
   output;
run;

data benchmark_output_deflator;
   set benchmark_output_deflator0_tmp1;
   array outputdefl_[10] outputdefl1998 - outputdefl2007;
   do i=1 to 10;
       outputdefl=outputdefl_[i];  
       year=1998+i-1;
	   output;
   end;
   keep year cic_adj outputdefl;
run;

proc sort data=benchmark_input_deflator ; by cic_adj year; run;
proc sort data=benchmark_output_deflator; by cic_adj year; run;

data benchmark_input_output_deflator;
   merge benchmark_input_deflator
         benchmark_output_deflator;
   by cic_adj year;
run;

*=====================================================================================;
*Brandt Rawski deflator available only up to 2006, extropolate to get 2007 br_deflator;
*(1) first difference.                                                                ;
*(2) fit a AR(3) model for the first difference.                                      ; 
*(3) forecast 2007 first difference                                                   ;
*(4) back out 2007 deflator level forecast                                            ;
*=====================================================================================;
data  br_capital_deflator_tmp1;
   set brandt_rawski_capital_deflator0;
   del_br_deflator=dif1(br_deflator);
   del_br_deflator_lag1=lag1(del_br_deflator);
   del_br_deflator_lag2=lag2(del_br_deflator);
   del_br_deflator_lag3=lag3(del_br_deflator);
run;

proc reg data=br_capital_deflator_tmp1 outest=est noprint;
   model del_br_deflator=del_br_deflator_lag1 del_br_deflator_lag2 del_br_deflator_lag3;
run;
quit;

data br_capital_deflator_tmp2;
     set br_capital_deflator_tmp1 end=last;
	 if ^last then output;
	 if last then do;
        output;
		year          =2007;
		br_deflator    =.;
        del_br_deflator=.;    
        output;
	 end;
run;

proc score data=br_capital_deflator_tmp2
            out =br_capital_deflator_tmp3
			score=est
			type=parms;
	 var del_br_deflator_lag1 del_br_deflator_lag2 del_br_deflator_lag3;
run;

data br_capital_deflator;
   set br_capital_deflator_tmp3;
   br_deflator_pred=lag1(br_deflator)+model1;
   if year=2007 then br_deflator=br_deflator_pred;
   keep year br_deflator;
run;

proc sql;
   create table deflator  as
   select a.*,
          b.br_deflator
   from benchmark_input_output_deflator as a,
        br_capital_deflator             as b
   where a.year=b.year;
quit;

*========================================================================================================================;                                                                                         ;
*Read in the firm data, recode ownership, realign industry classification (cic_adj of Brandt), include manufacturing only; 
*========================================================================================================================;
%let var_lst =labor total_wage benefit insurance_job intermediate output value_added vat fix_asset;

%include "D:\china project\export_import_decision_and_performance\data_realignment.sas"/source2;


proc sql;
   create table firm_tmp1  as
   select a.*,
          b.inputdefl,
          b.outputdefl, 
          b.br_deflator
   from firm as a,
        deflator  as b
   where a.year   =b.year   and 
         a.cic_adj=b.cic_adj ;
quit;

*==================================================================;
*here you can consider to do data smoothing, skip this step for now;
*==================================================================;


*=========================================================================;
*I:Calculate firm level TFP, using Brandt et al. deflators and concordance;
*First create input data for tfp estimation                               ;
*(a) saved to a sas dataset that will be transformed to stata dataset for ;
*    tfp estimation using LP method (see tfp.do file in the directory     ;
*    D:\china project\tfp_calculation\stata)                              ;
*(b) apply OLS estimate TFP                                               ;
*(c) merge a version of real capital stock created by using Brandt et al  ;
*    method in tfp_calculation_brandt_et_al.sas                           ;
*=========================================================================;
data tfp_input_data;
   set firm_tmp1; 
   output_real=output*100/outputdefl;
   if vat>0 then value_added_real=(output+vat)*100/outputdefl - intermediate*100/inputdefl;
       else value_added_real=output*100/outputdefl - intermediate*100/inputdefl;
   intermediate_real=intermediate*100/inputdefl;
   fix_asset_real=fix_asset*100/br_deflator;
   if output_real>0 then   l_output_real  =log(output_real);
      else  l_output_real =.;
   if value_added_real>0 then  l_value_added_real=log(value_added_real);
	  else l_value_added_real=.;
   if labor>0 then  l_labor=log(labor);
      else l_labor=.;
   if intermediate_real>0 then l_intermediate_real=log(intermediate_real);
      else l_intermediate_real=.;
   if fix_asset_real>0 then l_capital_real=log(fix_asset_real);
      else l_capital_real=.;
   keep frdm_panel_new name year cic_adj 
       output_real        l_output_real 
       value_added_real   l_value_added_real 
       labor              l_labor 
        intermediate_real l_intermediate_real 
       fix_asset_real     l_capital_real;
run;

data  l_capital_real_brandt;
   set output.tfp_data_for_stata;
   keep frdm_panel_new year l_capital_real_brandt;
run;

proc sort data=tfp_input_data       ; by frdm_panel_new year; run;
proc sort data=l_capital_real_brandt; by frdm_panel_new year; run;
data output.tfp_input_data;
   merge tfp_input_data
         l_capital_real_brandt;
   by frdm_panel_new year;
run;

proc sort data=output.tfp_input_data out=tfp_input_data; by cic_adj; run;

proc reg data=tfp_input_data;
    model l_value_added_real=l_labor l_capital_real /* l_intermediate_real */;
    output out=tfp_ols_value_added (keep=frdm_panel_new year tfp_ols_value_added cic_adj) residual=tfp_ols_value_added;
	by cic_adj;
	title "l_value_added_real=l_labor l_capital_real";
run;
quit;

proc reg data=tfp_input_data;
    model l_output_real=l_labor l_capital_real l_intermediate_real ;
    output out=tfp_ols_output (keep=frdm_panel_new year tfp_ols_output cic_adj) residual=tfp_ols_output;
	by cic_adj;
	title "l_output_real=l_labor l_capital_real l_intermediate_real";
run;
quit;

proc sort data=tfp_ols_value_added nodupkey; by frdm_panel_new year; run;  *27 duplicates, checed ok;
proc sort data=tfp_ols_output      nodupkey; by frdm_panel_new year; run;  *27 duplicates, checed ok;

data tfp_ols_value_added;
   set tfp_ols_value_added;
   tfp_ols_value_added=exp(tfp_ols_value_added);  *transferred back to level;
   keep frdm_panel_new year tfp_ols_value_added;
run;

data tfp_ols_output;
   set tfp_ols_output;
   tfp_ols_output=exp(tfp_ols_output); *transferred back to level;
   keep frdm_panel_new year tfp_ols_output;
run;

data tfp_ols;
   merge tfp_ols_value_added
         tfp_ols_output;
   by frdm_panel_new year;
run;


*=================================================================;
* In the background:                                              ;
*(1) Transfer tfp_input_data into stata dataset tfp_input_data.dta;
*(2) use stata code tfp.do to apply LP methodology to estimate tfp;
*    the result is in level                                       ;
*(3) notice tfp_lp_output and tfp_lp_value_added were in levels   ;
*    transformed here to make them in log                         ;
*=================================================================;
libname stata "D:\china project\tfp_calculation\stata";
proc sort data=stata.tfp9807            out=tfp9807           ; by id year; run;
proc sort data=stata.tfp_lp_output      out=tfp_lp_output     ; by id year; run;
proc sort data=stata.tfp_lp_value_added out=tfp_lp_value_added; by id year; run;

data tfp_lp (rename=(frdm_panel_new1=frdm_panel_new));
   length frdm_panel_new1 $20.;
   merge tfp9807
         tfp_lp_output
         tfp_lp_value_added;
   by id year;
   frdm_panel_new1=frdm_panel_new;
   drop frdm_panel_new;
run;

proc sort data=tfp_lp ; by frdm_panel_new year; run;
proc sort data=tfp_ols; by frdm_panel_new year; run;

data tfp_data;
   merge tfp_lp
         tfp_ols;
   by frdm_panel_new year;
   tfp_lp_output     =log(tfp_lp_output)     ;
   tfp_lp_value_added=log(tfp_lp_value_added);
run;

*======================================================================================;
*II: Create two productivity measures used in Tybout (QJE 1998, page 916) & others     ;
*   (a): labor productivity, log(LAB)                                                  ;
*   (b): average variable cost, log(AVC)                                               ;
*   *Where:                                                                            ;
*   AVC=TVC/(nominal output)=(total_wage+benefit+insurance_job+intermediate)/output    ;
*   LAB=real output/number of works=(output*100)/(outputdefl*labor)                    ; 
*       Errors of the regression equations respective to log(AVC) and log(LAB)(see page;
*       916) will be used as the indices of deviation from time and industry-specific  ;
*       productivity norms                                                             ;  
*III: Using same methodology in II, create additional productivity measure: log of real;
*     value added per worker=l_vad=log[(real output - real intermediate inputs)/labor] ;
*======================================================================================;

data firm_tmp2;
   set firm_tmp1; 
   tvc=total_wage+ benefit+ insurance_job+intermediate;
   if output>0 and tvc>0 then avc=tvc/output;
      else avc=.;
   output_real=output*100/outputdefl;
   fix_asset_real=fix_asset*100/br_deflator;
   if output>0 and labor>0 then lab=output*100/(outputdefl*labor);
      else lab=.;
   l_lab=log(lab);
   l_avc=log(avc);

   if vat>0 then value_added_real=(output+vat)*100/outputdefl - intermediate*100/inputdefl;
       else value_added_real=output*100/outputdefl - intermediate*100/inputdefl;
 
   if value_added_real>0 then l_vad=log(value_added_real);
      else l_vad=.;

   if fix_asset_real>0 then K=fix_asset_real;
      else K=.;
   l_K=log(K);
   l_K_sq=l_K*l_K;
   Age=year-open_year + 1;
   if age>=100 or age<=0 then age=.;
   l_age=log(age);
   l_age_sq=l_age*l_age;
  keep frdm_panel_new year l_lab l_avc l_vad l_K l_K_sq l_age l_age_sq ownership area_code cic_adj;
run;

*=========================================;
*Create year, province, and sector dummies;
*=========================================;
data firm_tmp3;
   set firm_tmp1;
   length province_code sector_code $2.;
   if area_code^=. then do;
      province_code=substr(compress(put(area_code,8.)),1,2);
   end;
   sector_code=substr(compress(put(cic_adj,4.)),1,2);
   keep frdm_panel_new year province_code sector_code ownership;
run;



ods listing close;
ods output onewayfreqs=sector_code;
proc freq data=firm_tmp3;
    table sector_code;
run;

ods output onewayfreqs=province_code;
proc freq data=firm_tmp3;
    table province_code;
run;
ods listing;

data _null_;
   format province_code_lst $char32767.;
   retain province_code_lst  '';
   retain province_code_lst_n  0 ;
   set province_code end=last;
   if province_code^="." then do;
       province_code_lst=trim(province_code_lst)||' '||(province_code);
	   province_code_lst_n+1;
   end;
   if last then do;
      call symput("province_code_lst", trim(province_code_lst));
      call symput("province_code_lst_n", trim(province_code_lst_n));
   end;
run;

data _null_;
   format sector_code_lst $char32767.;
   retain sector_code_lst  '';
   retain sector_code_lst_n  0 ;
   set sector_code end=last;
       sector_code_lst=trim(sector_code_lst)||' '||(sector_code);
	   sector_code_lst_n+1;
   if last then do;
      call symput("sector_code_lst", trim(sector_code_lst));
      call symput("sector_code_lst_n", trim(sector_code_lst_n));
   end;
run;

data firm_tmp3;
   set firm_tmp3;
   array province_code_lst_[1:&province_code_lst_n] _temporary_ (&province_code_lst);
   array province_code_lst_name_[1:&province_code_lst_n] province1-province%left(&province_code_lst_n);
   array sector_code_lst_[1:&sector_code_lst_n] _temporary_ (&sector_code_lst);
   array sector_code_lst_name_[1:&sector_code_lst_n] sector1 - sector%left(&sector_code_lst_n);
   array year_code_lst_[1998:2007] _temporary_ (1998 1999 2000 2001 2002 2003 2004 2005 2006 2007);
   array year_code_lst_name_[1998:2007] year1998 - year2007;

   do i=1 to &province_code_lst_n;
      province_code_lst_name_[i]=0;
   end;
   do j=1 to &province_code_lst_n;
      if province_code=province_code_lst_[j] then province_code_lst_name_[j]=1;
   end;

   do l=1 to &sector_code_lst_n;
      sector_code_lst_name_[l]=0;
   end;
   do m=1 to &sector_code_lst_n;
      if sector_code=sector_code_lst_[m] then sector_code_lst_name_[m]=1;
   end;
   do n=1998 to 2007;
      year_code_lst_name_[n]=0;
   end;
   do o=1998 to 2007;
      if year=year_code_lst_[o] then year_code_lst_name_[o]=1;
   end;
   dom=0;  
   fie=0; 
   hmt=0;  
   if upcase(ownership)="DOM" then dom=1;
   if upcase(ownership)="FIE" then fie=1;
   if upcase(ownership)="HMT" then hmt=1;
   drop province_code sector_code ownership i j l m n o;
run;

proc sort data=firm_tmp2 tagsort; by frdm_panel_new year; run;
proc sort data=firm_tmp3 tagsort; by frdm_panel_new year; run;

data firm_tmp3;
   merge firm_tmp2
         firm_tmp3;
   by frdm_panel_new year; 
run;  

%macro productivity(dependent=);
proc reg data=firm_tmp3;
    model  &dependent = l_age  l_age_sq l_k l_k_sq province2-province31 sector2-sector29 year1999-year2007 fie hmt;
	output out=&dependent    (keep=frdm_panel_new year &dependent &dependent._residual) RESIDUAL=&dependent._residual;
	title "&dependent =l_age  l_age_sq l_k l_k_sq province2-province31 sector2-sector29 year1999-year2007 fie hmt";
run;
quit;

%mend;

%productivity(dependent=l_avc);
%productivity(dependent=l_lab);
%productivity(dependent=l_vad);

proc sort data=l_lab    nodupkey; by frdm_panel_new year; run;  *27 duplicate keys deleted, checked no problem;
proc sort data=l_avc    nodupkey; by frdm_panel_new year; run; 
proc sort data=l_vad    nodupkey; by frdm_panel_new year; run;
proc sort data=tfp_data nodupkey; by frdm_panel_new year; run;

data output.productivity_measures;
   merge l_lab
         l_avc
		 l_vad
         tfp_data;
   by frdm_panel_new year;
   keep frdm_panel_new year cic_adj tfp_ols_value_added tfp_ols_output tfp_lp_value_added tfp_lp_output l_lab l_lab_residual l_avc l_avc_residual l_vad l_vad_residual;
run;




























